@@ -25,7 +25,7 @@ import zipfile
 
 from PyInstaller import compat
 from PyInstaller import log as logging
-from PyInstaller.compat import EXTENSION_SUFFIXES, is_darwin, is_win, is_linux
+from PyInstaller.compat import EXTENSION_SUFFIXES, is_aix, is_darwin, is_win, is_linux
 from PyInstaller.config import CONF
 from PyInstaller.exceptions import InvalidSrcDestTupleError
 from PyInstaller.utils import misc
@@ -238,6 +238,11 @@ def process_collected_binary(
         if is_darwin:
             # The default strip behavior breaks some shared libraries under macOS.
             strip_options = ["-S"]  # -S = strip only debug symbols.
+        elif is_aix:
+            # Set -X32_64 flag to have strip transparently process both 32-bit and 64-bit binaries, without user having
+            # to set OBJECT_MODE environment variable prior to the build. Also accommodates potential mixed-case
+            # scenario, for example a 32-bit utility program being collected into a 64-bit application bundle.
+            strip_options = ["-X32_64"]
 
         cmd = ["strip", *strip_options, cached_name]
         logger.info("Executing: %s", " ".join(cmd))
@@ -253,8 +258,16 @@ def process_collected_binary(
             )
             logger.debug("Output from strip command:\n%s", p.stdout)
         except subprocess.CalledProcessError as e:
-            logger.warning("Failed to run strip on %r!", cached_name, exc_info=True)
-            logger.warning("Output from strip command:\n%s", e.stdout)
+            show_warning = True
+
+            # On AIX, strip utility raises an error when ran against already-stripped binary. Catch the corresponding
+            # message (`0654-419 The specified archive file was already stripped.`) and suppress the warning.
+            if is_aix and "0654-419" in e.stdout:
+                show_warning = False
+
+            if show_warning:
+                logger.warning("Failed to run strip on %r!", cached_name, exc_info=True)
+                logger.warning("Output from strip command:\n%s", e.stdout)
         except Exception:
             logger.warning("Failed to run strip on %r!", cached_name, exc_info=True)
 
@@ -485,11 +498,18 @@ def format_binaries_and_datas(binaries_or_datas, workingdir=None):
 
         # Normalize paths.
         src_root_path_or_glob = os.path.normpath(src_root_path_or_glob)
-        if os.path.isfile(src_root_path_or_glob):
+
+        # If given source path is a file or directory path, pass it on.
+        # If not, treat it as a glob and pass on all matching paths. However, we need to preserve the directories
+        # captured by the glob - as opposed to collecting their contents into top-level target directory. Therefore,
+        # we set a flag which is used in subsequent processing to distinguish between original directory paths and
+        # directory paths that were captured by the glob.
+        if os.path.isfile(src_root_path_or_glob) or os.path.isdir(src_root_path_or_glob):
             src_root_paths = [src_root_path_or_glob]
+            was_glob = False
         else:
-            # List of the absolute paths of all source paths matching the current glob.
             src_root_paths = glob.glob(src_root_path_or_glob)
+            was_glob = True
 
         if not src_root_paths:
             raise SystemExit(f'ERROR: Unable to find {src_root_path_or_glob!r} when adding binary and data files.')
@@ -514,7 +534,12 @@ def format_binaries_and_datas(binaries_or_datas, workingdir=None):
                     #   "/top" from "/top/dir").
                     # * Normalizing the result to remove redundant relative paths (e.g., removing "./" from
                     #   "trg/./file").
-                    trg_dir = os.path.normpath(os.path.join(trg_root_dir, os.path.relpath(src_dir, src_root_path)))
+                    if was_glob:
+                        # Preserve directories captured by glob.
+                        rel_dir = os.path.relpath(src_dir, os.path.dirname(src_root_path))
+                    else:
+                        rel_dir = os.path.relpath(src_dir, src_root_path)
+                    trg_dir = os.path.normpath(os.path.join(trg_root_dir, rel_dir))
 
                     for src_file_basename in src_file_basenames:
                         src_file = os.path.join(src_dir, src_file_basename)
